import copy
from time import sleep
from rich.align import Align
from rich.live import Live
from rich.panel import Panel
from rich.text import Text

try:
    from utils import clear, GoBack
except ModuleNotFoundError:
    from thabox.utils import clear, GoBack

height: list = ['bottom', 'middle', 'top']

team_name: list = [
    '[purple]--------------------[/purple][magenta]This was built by:[/magenta]['
    'purple]--------------------[/purple] ',
    Text.assemble((r":                |$$$$$$$$| |$$| |$$| |$$$$$$|               :", 'bold purple'),
                  justify='center'),
    Text.assemble((r":                   |$$|    |$$| |$$| |$|                    :", 'bold purple'),
                  justify='center'),
    Text.assemble((r":                   |$$|    |$$$$$$$| |$$$$|                 :", 'bold purple'),
                  justify='center'),
    Text.assemble((r":                   |$$|    |$$| |$$| |$|                    :", 'bold purple'),
                  justify='center'),
    Text.assemble((":                   |$$|    |$$| |$$| |$$$$$$|               :\n", 'bold purple'),
                  justify='center'),

    Text.assemble((r":          |$| |$$$$$| |$$$$$| |$$$$$| |$$$$$| |$$$$$\       :", 'bold magenta'),
                  justify='center'),
    Text.assemble((r":          |$| |$$ $$|    /$$/    /$$/ |$|     |$$| \$\      :", 'bold magenta'),
                  justify='center'),
    Text.assemble((r":          |$| |$$$$$|   /$$/    /$$/  |$$$|   |$$| |$|      :", 'bold magenta'),
                  justify='center'),
    Text.assemble((r":       ___|$| |$| |$|  /$$/    /$$/   |$|     |$$| /$/      :", 'bold magenta'),
                  justify='center'),
    Text.assemble((":      |$$$$$| |$| |$| |$$$$$| |$$$$$| |$$$$$| |$$$$$/       :\n", 'bold magenta'),
                  justify='center'),

    Text.assemble((r":    |$| |$$$$$| |$$$$$\  |$$$$\   /$$$$$\  |$$$$$|  /$$$$$| :", 'bold purple'),
                  justify='center'),
    Text.assemble((r":    |$| |$|     |$$  $$| |$  $$| |$$   $$| |$$ $$| |$$\     :", 'bold purple'),
                  justify='center'),
    Text.assemble((r":    |$| |$$$|   |$$$$$/  |$$$$/  |$$   $$| |$$$$$|  \$$$$$\ :", 'bold purple'),
                  justify='center'),
    Text.assemble((r": ___|$| |$|     |$| \$\  |$  $$\ |$$   $$| |$| |$|      /$$|:", 'bold purple'),
                  justify='center'),
    Text.assemble((r":|$$$$$| |$$$$$| |$|  \$\ |$$$$$/  \$$$$$/  |$| |$|  |$$$$$/ :", 'bold purple'),
                  justify='center'),
    Text.assemble((r":____________________________________________________________:", 'bold purple'),
                  justify='center'),
]

jazzed_jerboas: list = [
    '\n\n[green]Teamleader| Discord: StormedPanda#9999 | GitHub: tomheaton | aka Tom |[/]\n\n',
    '\n\n[bold yellow]Teammember| Discord: Miklath#1000 | GitHub: MikeNoCap | aka Mike |[/]\n\n',
    '\n\n[bold yellow]Teammember | Discord: HiPeople21#6968 | GitHub: HiPeople21 |[/]\n\n',
    '\n\n[bold yellow]Teammember | Discord: Mega#6949 | GitHub: TahU28 |[/]\n\n',
    '\n\n[bold yellow]Teammember | Discord: Darling#9795 | GitHub: ryoflux |[/]\n\n',
    '\n\n[bold yellow]Teammember | Discord: babis99#8888 | GitHub: b-a-b-i-s |[/]\n\n']

message: list = [
    'We are very thankfull for getting to join this event. Special thanks to the Python Discord event-team\nfor taking their time off to manage and host this event.',
    '',
    'Placeholder'
]

rows_ = ["\n", "\n", "\n", "[bold white]-----------------------Credits-----------------------[/]", "\n",
         "\n", "\n", "[bold white]There will be stops along this ride so you can take your time to read.[/]"]
for i in range(9):
    rows_.append("")
for i in team_name:
    rows_.append(i)
for i in range(3):
    rows_.append("")
for i in jazzed_jerboas:
    rows_.append(i)
for i in range(12):
    rows_.append("")

for i in range(14):
    rows_.append("")
rows_.append("[bold blue]ThaBox was made as a submission for Python Discord's code-jam[/]")
rows_.append("[bold blue]and was developed by The Jazzed Jerboa's over the timespan of a week. [/]")
for i in range(2):
    rows_.append("")
rows_.append("[bold blue]We are very thankful for getting to join this event (Python code-jam Summer 2021). " \
             "Special thanks to the Python Discord event-team [/]")
rows_.append("[bold blue]for taking their time off to manage and host this event. [/]")
for i in range(2):
    rows_.append("")
rows_.append("[bold blue]Please don't look at The Jazzed Jerboa's as a team that submitted a project for the " \
             "Python code-jam Summer 2021,[/]")
rows_.append("[bold blue]but rather a team of wonderful people that shares the same interests. " \
             "Being a part of the team gives you a feeling that you really belong somewhere.[/]")
for i in range(2):
    rows_.append("")
rows_.append("[bold blue]During this event we have come a bit closer to mastering the art of collaboration.[/]")
rows_.append("[bold blue]It was super fun developing this! [/]")
for i in range(2):
    rows_.append("")
rows_.append("[bold blue]It's been a hell of a ride. [/]")
rows_.append("[bold blue]Enjoy our project! [/]")
for i in range(5):
    rows_.append("")

new_line = '\n'


def credits(console) -> None:
    """
    Displays the credits
    """
    with console.screen() as screen:
        rule: str = '[purple]--------------------[/purple][magenta]This was built by:[/magenta][' \
                    'purple]--------------------[/purple] '
        clear()
        text: Text = Align.center(
            Text.from_markup(
                f"{rule}", justify="center"),
            vertical=height[0],
        )
        screen.update(Panel(text, style="bold magenta", border_style="red"))
        sleep(0.5)

        for j in range(len(team_name)):
            console.clear()
            text: Text = Align.center(
                Text.from_markup(
                    f"{rule}\n{new_line.join([str(i) for i in team_name[:j]])}", justify="center"),
                vertical=height[0],
            )
            screen.update(Panel(text, style="bold magenta", border_style="red"))

        for j in range(len(jazzed_jerboas) + 1):
            clear()
            text: Text = Align.center(
                Text.from_markup(
                    f"{rule}\n{new_line.join([str(i) for i in team_name])}\n{new_line.join([str(i) for i in jazzed_jerboas[:j]])}",
                    justify="center"),
                vertical=height[0],
            )
            screen.update(Panel(text, style="bold magenta", border_style="red"))
            sleep(0.5)

        clear()
        text: Text = Align.center(
            Text.from_markup(
                f"{rule}\n{new_line.join([str(i) for i in team_name])}\n{new_line.join([str(i) for i in jazzed_jerboas])}",
                justify="center"),
            vertical=height[1],
        )
        screen.update(Panel(text, style="bold magenta", border_style="red"))
        sleep(0.5)

        clear()
        text: Text = Align.center(
            Text.from_markup(
                f"{rule}\n{new_line.join([str(i) for i in team_name])}\n{new_line.join([str(i) for i in jazzed_jerboas])}",
                justify="center"),
            vertical=height[2],
        )
        screen.update(Panel(text, style="bold magenta", border_style="red"))
        sleep(0.5)

        clear()
        text: Text = Align.center(
            Text.from_markup(
                f"\n{new_line.join([str(i) for i in team_name])}\n{new_line.join([str(i) for i in jazzed_jerboas])}",
                justify="center"),
            vertical=height[2],
        )
        screen.update(Panel(text, style="bold magenta", border_style="red"))
        sleep(0.5)

        for j in range(len(team_name)):
            clear()
            text: Text = Align.center(
                Text.from_markup(
                    f"{new_line.join([str(i) for i in team_name[j:]])}\n{new_line.join([str(i) for i in jazzed_jerboas])}",
                    justify="center"),
                vertical=height[2],
            )
            screen.update(Panel(text, style="bold magenta", border_style="red"))
            sleep(0.5)

        for j in range(len(jazzed_jerboas) + 1):
            clear()
            text: Text = Align.center(
                Text.from_markup(
                    f"{new_line.join([str(i) for i in jazzed_jerboas[j:]])}", justify="center"),
                vertical=height[2],
            )
            screen.update(Panel(text, style="bold magenta", border_style="red"))
            sleep(0.5)
        sleep(8)

        clear()
        text: Text = Align.center(
            Text.from_markup(
                "", justify="center"),
            vertical=height[2],
        )
        screen.update(Panel(text, style="bold magenta", border_style="red"))
        sleep(0.5)

        for j in range(len(message) + 1):
            clear()
            text: Text = Align.center(
                Text.from_markup(
                    f"{new_line.join([str(i) for i in message[:j]])}", justify="center"),
                vertical=height[0],
            )
            screen.update(Panel(text, style="bold magenta", border_style="red"))
            sleep(1)

        clear()
        text: Text = Align.center(
            Text.from_markup(
                f"{new_line.join([str(i) for i in message])}", justify="center"),
            vertical=height[1],
        )
        screen.update(Panel(text, style="bold magenta", border_style="red"))
        sleep(1)

        for j in range(len(message)):
            clear()
            text: Text = Align.center(
                Text.from_markup(
                    f"{new_line.join([str(i) for i in message[j:]])}", justify="center"),
                vertical=height[2],
            )
            screen.update(Panel(text, style="bold magenta", border_style="red"))
            sleep(1)

        clear()
        text: Text = Align.center(
            Text.from_markup(
                "", justify="center"),
            vertical=height[2],
        )
        screen.update(Panel(text, style="bold magenta", border_style="red"))
        sleep(0.5)
        clear()


def credits_rework() -> None:
    rows = copy.deepcopy(rows_)
    with Live("", refresh_per_second=9) as screen:

        loop_times = 0  # Keep track of where in rows we are so we can stop at a specific point

        while len(rows) != 0:
            loop_times += 1
            rows.pop(0)

            scroll_through = Align.center(
                Text.from_markup(
                    "".join(
                        [x + "\n" if not isinstance(x, Text) else Text.assemble((x))._text[1] + "\n" for x in rows]),
                    justify="center"
                )
            )

            screen.update(Panel(scroll_through, border_style="bold red", style="bold magenta"))
            if loop_times == 3:
                for i in range(6):
                    screen.update(Panel(scroll_through, border_style="bold yellow", style="bold magenta"))
                    sleep(1)
            if loop_times == 37:
                for i in range(10):
                    screen.update(Panel(scroll_through, border_style="bold yellow", style="bold magenta"))
                    sleep(1)
            if loop_times == 65:
                for i in range(32):
                    screen.update(Panel(scroll_through, border_style="bold yellow", style="bold magenta"))
                    sleep(1)
            if loop_times == 78:
                for i in range(9):
                    screen.update(Panel(scroll_through, border_style="bold yellow", style="bold magenta"))
                    sleep(1)

            sleep(0.4)
    clear()


def skip_credits():
    raise GoBack



import pickle
import random
import re
import string
import time

from hashlib import sha256
from rich.console import Console
from rich.panel import Panel
from rich.prompt import Prompt
from rich.text import Text
from rich.markup import escape
import asyncio

console = Console()




import os
import time

from rich.console import Console
from rich.markdown import Markdown
from rich.panel import Panel
from rich.prompt import Prompt
from rich.text import Text

console: Console = Console()


class Preferences:
    """
    This needs to be passed onto the 'User' class as it sets the default preferences:
    'sampleuser = User("samplename", "password", default_preference)'

    default_preference = {
        "Name Colour" : "bold cyan",
        "Message Colour" : "bold white",
        "Border Colour" : "bold magenta"
    }

    To change the preference, this is used:
    'sampleuser.preferences = Preferences.settings(sampleuser.preferences)'
    *Note: this only allows for one settings to be changed, so a while loop would need
    to be used to allow multiple settings be to changed.
    """

    def __init__(self):
        self.preference_dict = default_preference = {
            "Name Colour": "bold cyan",
            "Message Colour": "bold white",
            "Border Colour": "bold purple",
            "Message Border Colour": "bold magenta",
        }

    def template_message(self) -> None:
        """
        Function which prints out a preview of how the users box is going to look.
        """
        message_preview = f"""[{self.preference_dict["Message Border Colour"]}]┌────────────────────────────────┐[/]
[{self.preference_dict["Message Border Colour"]}]│{"Message from" : ^32}│[/]
[{self.preference_dict["Message Border Colour"]}]│[/][{self.preference_dict["Name Colour"]}]{"Preview" : ^32}[/][{self.preference_dict["Message Border Colour"]}]│[/]
[{self.preference_dict["Message Border Colour"]}]├────────────────────────────────┤[/]
[{self.preference_dict["Message Border Colour"]}]│[/][{self.preference_dict["Message Colour"]}]This is what a message looks[/]    [{self.preference_dict["Message Border Colour"]}]│[/]
[{self.preference_dict["Message Border Colour"]}]│[/][{self.preference_dict["Message Colour"]}]like with your current[/]          [{self.preference_dict["Message Border Colour"]}]│[/]
[{self.preference_dict["Message Border Colour"]}]│[/][{self.preference_dict["Message Colour"]}]preferences.[/]                    [{self.preference_dict["Message Border Colour"]}]│[/]
[{self.preference_dict["Message Border Colour"]}]└────────────────────────────────┘[/]"""
        console.print(Panel(message_preview, border_style=self.preference_dict["Border Colour"]))

    def settings(self):
        """
        Function which allows the user to change thier settings. Allows hex and named
        colour values and returns then updated preference.
        """
        while True:
            Preferences.template_message(self)
            val_to_be_changed: str = make_style_prompt(prompt_msg="What would you like to change?",
                                                       choices=["Border Colour", "Name Colour", "Message Colour",
                                                                "Message Border Colour", "[red]Exit preferences[/]"],
                                                       main_style=self.preference_dict["Message Border Colour"],
                                                       frame_border_style=self.preference_dict["Border Colour"])
            clear()
            if val_to_be_changed == "[red]Exit preferences[/]":
                raise GoBack

            else:
                while True:
                    console.print(Panel("🎨 Name a color or enter a hex-value",
                                        border_style=self.preference_dict["Message Border Colour"]))
                    colour: str = Prompt.ask(Text.assemble(("╰>", self.preference_dict["Message Colour"])))
                    try:
                        test: str = console.print("", style=colour)
                    except:
                        clear()
                        console.print("❌   Please enter a valid colour!\n")
                        time.sleep(2.2)
                        clear()
                    else:
                        previous_setting: str = self.preference_dict[val_to_be_changed]
                        self.preference_dict[val_to_be_changed] = "bold " + str(colour)
                        console.print("✔️   Updated successfully!\n")
                        time.sleep(2.2)
                        # update_userfiles()
                        clear()
                        break


class User:
    def __init__(self, username: str, paswrd: str, preferences: Preferences):
        self.username = username
        self.hashed_pass = paswrd
        self.preferences = preferences
        self.friends = []


class GoBack(Exception):
    pass


def make_style_prompt(choices: list, default: str = None, prompt_msg: str = "Would you like to:",
                      main_style: str = "none",
                      frame_style: str = "none",
                      frame_border_style: str = "none") -> str:
    """
    Prompts user in a cool way and retrieves what the user picked.

    :param choices: A list of choices.
    :param default: The value that gets returned if user doesn't type anything in.
    :param prompt_msg: The message being printed before the choices.
    :param main_style: The main theme/color of the prompt.
    :param frame_style: The theme/color for the text in the panels.
    :param frame_border_style: The theme/color for the frame in the panels.
    :return: str value of final style choice the user selected.
    """
    choices = [str(i) for i in choices]
    if default is not None:
        # Get index of default so it can be set to the default choice.
        default_index = str(choices.index(default) + 1)
    else:
        default_index = None

    choices_styled = []
    c = 0
    for i in choices:
        c += 1
        choices_styled.append(Panel(f"{c}. {i}", style=frame_style, border_style=frame_border_style))

    console.print(Panel(prompt_msg.replace("\n", ""), style=main_style, border_style=main_style))
    for i in choices_styled:
        console.print(i)

    choice_index = Prompt.ask(Text.assemble(("╰>", main_style)), choices=[str(x) for x in range(1, c + 1)],
                              default=default_index)
    choice_index = int(choice_index)
    return choices[choice_index - 1]


def clear() -> int:
    """
    Clears terminal
    """
    return os.system('cls' if os.name == 'nt' else 'clear')



import textwrap
from typing import List, TYPE_CHECKING
from rich import console
from rich.panel import Panel
from rich.text import Text
from rich.table import Table
from rich.console import Console
from rich.screen import Screen
from rich.live import Live
from rich.markup import escape
from rich.layout import Layout
from rich.prompt import Prompt
import time
import random
import asyncio
import keyboard

try:
    from utils import clear, User, Preferences
except ModuleNotFoundError:
    from thabox.utils import clear, User, Preferences

client_user = None


async def get_index_duplicates(lst, item) -> list:
    """
    :param lst: The list to search for an item or items in.
    :param item: The item to find indexes for in list.
    Like list.index(), but list.index() only returns one index. This function returns all the indexes of which an item appears.
    """
    return [i for i, x in enumerate(lst) if x == item]



menu_no_color = """╭────────────────────────────────╮
│                                │
│ $$$$$$$$\ $$\                  │
│ \__$$  __|$$ |                 │
│    $$ |   $$$$$$$\   $$$$$$\   │
│    $$ |   $$  __$$\  \____$$\  │
│    $$ |   $$ |  $$ | $$$$$$$ | │
│    $$ |   $$ |  $$ |$$  __$$ | │
│    $$ |   $$ |  $$ |\$$$$$$$ | │
│    \__|   \__|  \__| \_______| │
│                                │
│ $$$$$$$\                       │
│ $$  __$$\                      │
│ $$ |  $$ | $$$$$$\  $$\   $$\  │
│ $$$$$$$\ |$$  __$$\ \$$\ $$  | │
│ $$  __$$\ $$ /  $$ | \$$$$  /  │
│ $$ |  $$ |$$ |  $$ | $$  $$<   │
│ $$$$$$$  |\$$$$$$  |$$  /\$$\  │
│ \_______/  \______/ \__/  \__| │
│                                │
╰────────────────────────────────╯"""

# Create the colored logo-text manually because rich's Panel()
# function does not allow you to get the raw_output (the string that is actually being printed)
RED = '\033[91m'
ENDC = '\033[0m'
PURPLE = '\033[95m'
CYAN = '\033[96m'
box_logo_lines = [
    RED + "╭────────────────────────────────╮  " + ENDC,
    RED + "│                                │  " + ENDC,
    RED + "│" + ENDC + PURPLE + " $$$$$$$$\\ $$\\                  " + ENDC + RED + "│" + ENDC + "  ",
    RED + "│" + ENDC + PURPLE + " \\__$$  __|$$ |                 " + ENDC + RED + "│" + ENDC + "  ",
    RED + "│" + ENDC + PURPLE + "    $$ |   $$$$$$$\\   $$$$$$\\   " + ENDC + RED + "│" + ENDC + "  ",
    RED + "│" + ENDC + PURPLE + "    $$ |   $$  __$$\\  \\____$$\\  " + ENDC + RED + "│" + ENDC + "  ",
    RED + "│" + ENDC + PURPLE + "    $$ |   $$ |  $$ | $$$$$$$ | " + ENDC + RED + "│" + ENDC + "  ",
    RED + "│" + ENDC + PURPLE + "    $$ |   $$ |  $$ |$$  __$$ | " + ENDC + RED + "│" + ENDC + "  ",
    RED + "│" + ENDC + PURPLE + "    $$ |   $$ |  $$ |\\$$$$$$$ | " + ENDC + RED + "│" + ENDC + "  ",
    RED + "│" + ENDC + PURPLE + "    \\__|   \\__|  \\__| \\_______| " + ENDC + RED + "│" + ENDC + "  ",
    RED + "│                                │" + ENDC + "  ",
    RED + "│" + ENDC + CYAN + " $$$$$$$\\                       " + ENDC + RED + "│" + ENDC + "  ",
    RED + "│" + ENDC + CYAN + " $$  __$$\\                      " + ENDC + RED + "│" + ENDC + "  ",
    RED + "│" + ENDC + CYAN + " $$ |  $$ | $$$$$$\\  $$\\   $$\\  " + ENDC + RED + "│" + ENDC + "  ",
    RED + "│" + ENDC + CYAN + " $$$$$$$\\ |$$  __$$\\ \\$$\\ $$  | " + ENDC + RED + "│" + ENDC + "  ",
    RED + "│" + ENDC + CYAN + " $$  __$$\\ $$ /  $$ | \\$$$$  /  " + ENDC + RED + "│" + ENDC + "  ",
    RED + "│" + ENDC + CYAN + " $$ |  $$ |$$ |  $$ | $$  $$<   " + ENDC + RED + "│" + ENDC + "  ",
    RED + "│" + ENDC + CYAN + " $$$$$$$  |\\$$$$$$  |$$  /\\$$\\  " + ENDC + RED + "│" + ENDC + "  ",
    RED + "│" + ENDC + CYAN + " \\_______/  \\______/ \\__/  \\__| " + ENDC + RED + "│" + ENDC + "  ",
    RED + "│                                │  " + ENDC,
    RED + "╰────────────────────────────────╯  " + ENDC
]
menu_logo = "".join([x + "\n" if x != box_logo_lines[-1] else x for x in box_logo_lines])

console = Console()


async def render_menu_screen(rows: list) -> Text:
    """
    This function gets the Text-object that is shown to the user once in the main_menu or while in a box.

    :param rows: A list of strings that together make a box. (There must be at least 21 rows/items)
    :return: Text-object which can be printed with console.print().
    """

    logo_rows = menu_logo.split("\n")

    if len(rows) < 21:
        raise ValueError("The argument rows needs to contain at least 21 rows.")
    if len(rows) > 21:
        dummy_rows_to_add = len(rows) - len(logo_rows)
        dummy_rows = [Text.assemble(('-' * 34 + "  ", "red")) for _ in range(dummy_rows_to_add)]
        for i in range(dummy_rows_to_add):
            logo_rows.append(dummy_rows[i])

    new_rows = []
    for i in range(0, len(rows)):
        new_rows.append(Text.assemble(logo_rows[i], rows[i] + '\n'))
    return Text.assemble(*new_rows)


async def get_message_box_rows(message_box: list, user: User) -> list:
    message_box = [_ + "".join([" " for i in range(90 - len(_))]) for _ in message_box]

    while len(message_box) != 26:
        message_box.append("".join([" " for i in range(90)]))
        if len(message_box) == 26:
            break
        message_box.insert(0, "".join([" " for i in range(90)]))

    if len(message_box) != 26:
        raise ValueError("Box must contain 26 rows.")

    border_color = user.preferences.preference_dict["Border Colour"]
    new_rows = []

    new_rows.append(
        Text.assemble(("┌──────────────────────────────────────────────────────────────────────────────────────────┐",
                       border_color)))
    for i in message_box:
        new_rows.append(Text.assemble(("│", border_color), i, ("│", border_color)))
    new_rows.append(
        Text.assemble(("└──────────────────────────────────────────────────────────────────────────────────────────┘",
                       border_color)))
    return new_rows


async def get_message_box(msg_sender: User, message: str, stage: int) -> list:
    """Returns a list of strings which together assemble the message box that is being displayed in render_message."""

    name_color = msg_sender.preferences.preference_dict["Name Colour"]
    message_color = msg_sender.preferences.preference_dict["Message Colour"]
    border_color = msg_sender.preferences.preference_dict["Message Border Colour"]

    message_lines = textwrap.wrap(message, width=32)

    message_string = ""
    count = 0
    for _ in message_lines:
        count += 1
        if not count == len(message_lines):
            message_string += f"│{_ : <32}│" + "\n"
            continue
        message_string += f"│{_ : <32}│"

    if len(message_string.splitlines()) > 9:
        raise ValueError("Message too long!")

    msg_box = \
        f"┌────────────────────────────────┐\n" \
        f"│{'Message from' : ^32}│\n" \
        f"│{(msg_sender.username) : ^32}│\n" \
        f"├────────────────────────────────┤\n" \
        f"{message_string}\n" \
        f"└────────────────────────────────┘"

    lines = msg_box.splitlines()
    if stage < 34:
        a = [line[(34 - stage):34] for line in lines]
    if stage >= 34:
        a = ["".join([" " for _ in range(stage - 34)]) + line for line in lines]

    new = []

    c = 0
    for i in a:
        c += 1
        if c == 1 or c == 2 or c == 4:
            new.append(Text.from_markup(f"[{border_color}]{i}[/]"))
            continue
        if c == 3:
            list_username = list(msg_sender.username)
            list_line = list(i)
            if list_line.count("│") - list_username.count("│") == 2:  # Ignore │ in usernames.
                indexes = (await get_index_duplicates(list_line, "│"))
                list_line[indexes[0]] = f"[{border_color}]│[/][{name_color}]"
                list_line[indexes[-1]] = f"[/][{border_color}]│[/]"
                new.append(Text.from_markup("".join(list_line)))
            if list_line.count("│") - list_username.count("│") == 1:
                indexes = (await get_index_duplicates(list_line, "│"))
                list_line[indexes[-1]] = f"[/][{border_color}]│[/]"
                list_line.insert(0, f"[{name_color}]")
                new.append(Text.from_markup("".join(list_line)))
            continue
        if not c - 4 > len(message_lines):
            list_line = list(i)
            current_msg_line = list(message_lines[c - 5])
            if list_line.count("│") - current_msg_line.count("│") == 2:  # Ignore │ in usernames.
                indexes = (await get_index_duplicates(list_line, "│"))

                list_line[indexes[0]] = f"[{border_color}]│[/][{message_color}]"
                list_line[indexes[-1]] = f"[/][{border_color}]│[/]"

                new.append(Text.from_markup("".join(list_line)))
            if list_line.count("│") - current_msg_line.count("│") == 1:
                indexes = (await get_index_duplicates(list_line, "│"))

                if stage == 1:
                    list_line[0] = f"[{border_color}]│[/]"
                else:
                    list_line[0] = f"[{message_color}]"
                    list_line[indexes[-1]] = f" [{border_color}]│[/]"

                new.append(Text.from_markup("".join(list_line)))

        if c - 4 > len(message_lines):
            list_line = list(i)
            if "└" in list_line and "┘" in list_line:
                indexleft = (await get_index_duplicates(list_line, "└"))[0]
                indexright = (await get_index_duplicates(list_line, "┘"))[0]

                list_line[indexleft] = f"[{border_color}]└"
                list_line[indexright] = f"┘[/]"
                new.append(Text.from_markup("".join(list_line)))

            if "┘" in list_line and not "└" in list_line:
                index = (await get_index_duplicates(list_line, "┘"))[0]

                if stage == 1:
                    list_line[index] = f"[{border_color}]┘[/]"

                else:
                    list_line[0] = f"[{border_color}]─"
                    list_line[index] = f"┘[/]"
                new.append(Text.from_markup("".join(list_line)))

    return new


async def load_box_animation(user):
    string = menu_no_color
    string_rows = string.split("\n")
    rows_2D = []
    for j in range(0, len(string_rows[0])):
        rows_2D.append([_[j] for _ in string_rows])
            

    frames = []
    

    width = 0
    s = ""
    rows = []

    for i in rows_2D:
        width += 1
        if width == 1:
            for _ in i:
                rows.append(_)
                frames.append("".join(k+"\n" for k in rows))

        else:
            if width % 2 == 0:
                for j in list(range(0, len(rows)))[::-1]:
                    rows[j] += i[j]
                    frames.append("".join(k+"\n" for k in rows))
            else:   
                for j in range(0, len(rows)):
                    rows[j] += i[j]
                    frames.append("".join(k+"\n" for k in rows))

        frames.append("".join(k+"\n" for k in rows))

    

    
    with Live(refresh_per_second=10) as l:
        for i in frames:
            l.update(i)
            await asyncio.sleep(0.000001)
    clear()
    for i in range(0, len(rows)):
        rows[i] = box_logo_lines[i]
        for _ in rows:
            print(_)
        if i != len(rows):
            clear()    
    console.print(await render_menu_screen(["" for i in range(21)]))

async def render_message(message: str, user: User, message_show_time: int = 6, live=Live()) -> Text:
    fade_left_frames = []
    for _ in range(0, 63):
        message_box = (await get_message_box(user, message, _))
        frame = (await render_menu_screen(await get_message_box_rows(message_box, user)))
        fade_left_frames.append(frame)

    going_down_box = (await get_message_box(user, message, 62))
    going_down_frames = []

    message_size = len(going_down_box)

    for i in range(26 - message_size):
        going_down_box.insert(0, "".join([" " for ___ in range(90)]))
        going_down_frames.append(await render_menu_screen(await get_message_box_rows(going_down_box, user)))
    for i in range(message_size):
        going_down_box.insert(0, "".join([" " for ___ in range(90)]))
        going_down_box.pop(-1)
        going_down_frames.append(await render_menu_screen(await get_message_box_rows(going_down_box, user)))
    
    frame_time_in = 0.05
    frame_time_out = 0.05
    for i in fade_left_frames:
        live.update(Text.assemble(i, ("\nYou can send messages once messages are done displaying...")))
        await asyncio.sleep(frame_time_in)

    await asyncio.sleep(message_show_time)

    for i in going_down_frames:
        live.update(Text.assemble(i, ("\nYou can send messages once messages are done displaying...")))
        await asyncio.sleep(frame_time_in)
        await asyncio.sleep(frame_time_out)

    return live


async def message_demo(user: User):
    global client_user
    client_user = user
    console = Console()
    while True:
        with Live("") as live:
            live.update(
                await render_menu_screen(await get_message_box_rows(["".join(" " for i in range(90)) for j in range(26)], user)))
            mes = "This is a demo message..."
            live = await render_message(mes, user, live=live)
            await asyncio.sleep(100)
  

async def render_chat_rooms(rows: list, hover_on: int) -> Text:
    """
    This function gets the Text-object that is shown to the user once in the main_menu or while in a box.

    :param rows: A list of strings that together make a box. (There must be at least 21 rows/items)
    :return: Text-object which can be printed with console.print().
    """

    logo_rows = menu_logo.split("\n")

    dummy_rows_to_add = len(rows[:hover_on + 7]) - len(logo_rows)  # [:(hover_on + 7) * 4 + 1]

    dummy_rows = [Text.assemble(('-' * 34 + "  ", "red")) for _ in range(dummy_rows_to_add)]
    for i in range(dummy_rows_to_add):
        logo_rows.append(dummy_rows[i])

    new_rows = []

    for i in range(0, len(logo_rows)):
        try:
            new_rows.append(Text.assemble(logo_rows[i], rows[i] + '\n'))
        except IndexError:
            new_rows.append(Text.assemble(logo_rows[i] + '\n'))

    return Text.assemble(*new_rows)


async def prompt(user):
    console = Console()
    console.print(await render_menu_screen(await get_message_box_rows([""], user)))
    accepted = False
    while not accepted:
        a = Prompt.ask("Send a message")
        if a == "":
            console.print("[red]Can not send empty message[/]")
            continue
        if len(a) > 32*8:
            console.print("[red]Can not send message with more than {} characters.[/]".format(str(32*8)))
        accepted = True
    return a

if __name__ == "__main__":
    asyncio.run(load_box_animation(None))


import keyboard
import time
from rich import print
from rich.console import Console
from rich.prompt import Prompt
from rich.align import Align
import asyncio

try:
    from utils import clear, GoBack
except ModuleNotFoundError:
    from thabox.utils import clear, GoBack

console = Console()


async def help_navigation(currentpage, user):
    """
    Allows user to cycle through the help pages.
    """

    page_cycle: list = ["About", "Boxes", "Messages", "Pref"]
    currentpage: int = page_cycle.index(currentpage)

    if currentpage == 3:
        next: int = page_cycle[0]
        previous: int = page_cycle[2]

    elif currentpage == 0:
        next: int = page_cycle[1]
        previous: int = page_cycle[3]

    else:
        next: int = page_cycle[currentpage + 1]
        previous: int = page_cycle[currentpage - 1]

    console.print(
        "\n[bold white][bold red]![/]   Use [bold magenta] right arrow[/] or [bold magenta]left arrow[/] to navigate the page, or press [bold magenta]BACKSPACE[/] to return to Menu.[/]")

    # Adding delays between key presses so that the pages won't cyle through too fast.
    await asyncio.sleep(0.2)

    while True:
        if keyboard.is_pressed("left_arrow"):
            if previous == "About":
                await help_start_page(user)
                break

            elif previous == "Boxes":
                await help_boxes(user)
                break

            elif previous == "Messages":
                await help_messages(user)
                break

            elif previous == "Pref":
                await help_preference(user)
                break

        elif keyboard.is_pressed("right_arrow"):
            if next == "About":
                await help_start_page(user)
                break

            elif next == "Boxes":
                await help_boxes(user)
                break

            elif next == "Messages":
                await help_messages(user)
                break

            elif next == "Pref":
                await help_preference(user)
                break

        elif keyboard.is_pressed("backspace"):
            raise GoBack


async def help_start_page(user):
    if user is not None:
        main_style = user.preferences.preference_dict["Border Colour"]
    else:
        main_style = "bold cyan"
    clear()
    paragraph = "[bright_white]ThaBox is a social media platform in which the conversations " \
                "\nbetween users are for the users only. Not a single message is " \
                "\nsaved, giving you the utmost privacy. Chat with " \
                "\nstrangers in boxes, or have a blast with your friends " \
                "\nin boxes. " \
                "\n\n[/][bold grey58]This program was created for the 'Python Discord 2021 CodeJam'.\n[/]"

    paragraph = """
    [bright_white]ThaBox is a social media platform in which the conversations 
    between users are for the users only. Not a single message is
    saved, giving you the utmost privacy. Chat with
    strangers in boxes, or have a blast with your friends
    in boxes.
    
    [/][bold grey58]This program was created for the 'Python Discord 2021 CodeJam'.\n[/]
    """

    console.print((Align(f"""[{main_style}]
    ▄▀█ █▄▄ █▀█ █ █ ▀█▀
    █▀█ █▄█ █▄█ █▄█  █ [/]\n""", align="center", )))
    console.print((Align(paragraph, align="center")))
    await help_navigation('About', user)


async def help_boxes(user):
    if user is not None:
        main_style = user.preferences.preference_dict["Border Colour"]
    else:
        main_style = "bold cyan"
    clear()
    paragraph = "[bright_white]There is one type of box available." \
                "\When creating a box, it will check if a box of that name already exists, and" \
                "\nif it doesn't one will be created. However, if it does" \
                "\nexist, you will join the box and be able to start talking.\n[/]"
    paragraph = """
    [bright_white]There is one type of box available. 
    When creating a box, it will check if a box of that name already exists, and
    if it doesn't one will be created. However, if it does exist,
    you will join the box and be able to start talking.
    [/]
    """

    console.print((Align(f"""[{main_style}]
     █▄▄ █▀█ ▀▄▀ █▀▀ █▀
     █▄█ █▄█ █ █ ██▄ ▄█[/]\n""", align="center", )))
    console.print((Align(paragraph, align="center")))
    await help_navigation('Boxes', user)


async def help_messages(user):
    if user is not None:
        main_style = user.preferences.preference_dict["Border Colour"]
    else:
        main_style = "bold cyan"
    clear()
    paragraph = "[bright_white]Messages can be sent to another person in the same box, by simply"\
                "\ntyping it out when you receive the prompt. However"\
                "\nyou can only see the other people's messages after you"\
                "\nhave finished typing."\
                "\n\nMessages that you receive are available for some seconds."\
                "\nPast message are not stored and can't be "\
                "\nretrieved.\n[/]"
    paragraph = """
    [bright_white]Messages can be sent to another person in the same box, by simply
    typing it out when you receive the prompt. However you can only see the other 
    people's messages after you have finished typing.
    
    Messages that you receive are available for some seconds.
    message has been sent. Past message are not stored and can't be
    retrieved.
    [/]
    """

    console.print((Align(f"""[{main_style}]
    █▀▄▀█ █▀▀ █▀ █▀ ▄▀█ █▀▀ █▀▀ █▀
    █ ▀ █ ██▄ ▄█ ▄█ █▀█ █▄█ ██▄ ▄█[/]\n""", align="center", )))
    console.print((Align(paragraph, align="center")))
    await help_navigation("Messages", user)


async def help_preference(user):
    if user is not None:
        main_style = user.preferences.preference_dict["Border Colour"]
    else:
        main_style = "bold cyan"
    clear()
    paragraph = "[bright_white]The colour scheme of your box can be changed in the"\
                "\nPreferences page once logged in. A valid colour name or a hex-value (#xxxxxx)"\
                "\ncan be used to personalise different parts of your box. [/]\n"
    paragraph = """
    [bright_white]The colour scheme of your box can be changed in the
    Preferences page once logged in. A valid colour name or a hex-value (#xxxxxx)
    can be used to personalise different parts of your box. [/]
    
    """

    console.print((Align(f"""[{main_style}]
    █▀█ █▀█ █▀▀ █▀▀ █▀▀ █▀█ █▀▀ █▄ █ █▀▀ █▀▀ █▀
    █▀▀ █▀▄ ██▄ █▀  ██▄ █▀▄ ██▄ █ ▀█ █▄▄ ██▄ ▄█[/]\n""", align="center", )))
    console.print((Align(paragraph, align="center")))
    await help_navigation("Pref", user)




# Client
import asyncio
import socketio
import keyboard
from rich.console import Console
from rich.prompt import Prompt, Confirm, IntPrompt
from rich.panel import Panel
from rich.text import Text
from rich.live import Live
import sys
import os
import string
import random
import time
from hashlib import sha256

try:
    import utils
    import main as main_navigation
    from utils import clear, User, Preferences
    import rendering
except ModuleNotFoundError:
    import thabox.utils as utils
    import thabox.main as main_navigation
    from thabox.utils import clear, User, Preferences
    import thabox.rendering as rendering


class MessagePromptStop(Exception):
    pass


sio = socketio.AsyncClient(request_timeout=30)

console = Console()


CONNECTED: bool = False
SERVER_ADDRESS = ("http://localhost:8000", "http://thabox.asmul.net:8000")[1]

USERNAME = ""
ROOM = ""
ROOMS: list = []
ROOM_WORKS : bool = False
WAIT_FOR_INFO = False
LOGIN_DATA = 0

messages_to_show: list = []

printable_chars = list(string.printable.replace("\n", "").replace(" ", "").replace("    ", ""))
def add_salt(hash_no_salt):
    for i in range(9):
        hash_no_salt += random.choice(printable_chars)
    return hash_no_salt


def exit_anim():
    print_string = "[a]Hope[/][b] you[/][a] had[/][b] a[/][a] great[/][b] time![/]"
    s1 = print_string.replace("[a]", "[cyan]").replace("[b]", "[magenta]")
    s2 = print_string.replace("[a]", "[magenta]").replace("[b]", "[cyan]")


    frames = []

    extra = ""
    s = ""

    for i in range(30):
        x = s1 if i % 2 == 0 else s2
        frames.append(Panel(extra+s+x, height=32, width=90))
        s += " "
        extra += "\n"
    for i in range(30):
        x = s1 if i % 2 == 0 else s2
        s+=" "
        extra = extra[:-1]
        frames.append(Panel(extra+s+x, height=32, width=90))
    frames.append(Panel("", height=32, width=90))
    console = Console()
    os.system("cls" if os.name == "nt" else "clear")


    with Live(refresh_per_second=10) as l:
        for i in frames:
            time.sleep(0.15)
            l.update(i)

    sys.exit(1)


def remove_salt(hash_and_salt):
    back_hash_removed = hash_and_salt[:-9]
    return back_hash_removed


def hash_pass(passwrd: str):
    hashed = sha256(bytes(passwrd, "utf8")).hexdigest()
    return add_salt(hashed)


async def create_account(username: str, password: str):
    password = hash_pass(password)
    prefs = Preferences()
    user_ob = User(username=username, paswrd=password, preferences=prefs)
    await save_user(username, password, prefs.preference_dict)
    return user_ob


async def sign_up():
    global secrets
    progress_visual = []
    username_status = "working"
    while username_status != "ready":
        clear()
        console.print(Panel(Text.assemble(("Enter a username", "bold purple")),
                            style="bold purple", border_style="bold purple"))

        username = Prompt.ask(Text.assemble(("╰→", "bold red")))
        console.print("Checking username...", style="bold yellow")
        data = await get_user_data(username)

        if data is not None:
            clear()
            console.print(Panel(Text.assemble(("Username already taken", "bold purple")),
                                style="bold red", border_style="bold red"))
            await asyncio.sleep(2.1)
            clear()
            continue
        if " " in username:
            clear()
            console.print(Panel(Text.assemble(("Can not have space in username", "bold purple")),
                                style="bold red", border_style="bold red"))
            await asyncio.sleep(2.1)
            clear()
            continue
        if "[" in username or "]" in username:
            clear()
            console.print(Panel(Text.assemble(("Username must not contain square-brackets ('[', ']')",
                                               "bold cyan")), style="bold red", border_style="bold red"))
            await asyncio.sleep(3.1)
            clear()
            continue
        if len(username) < 4:
            clear()
            console.print(Panel(Text.assemble(("Username must be at least 4 characters long",
                                               "bold purple")), style="bold red", border_style="bold red"))
            await asyncio.sleep(2.1)
            clear()
            continue

        progress_visual.append(Panel(Text.assemble(("Username ☺", "bold green")),
                                     style="bold green", border_style="bold green"))
        progress_visual.append(Text.assemble(("╰→", "bold red"), (username, "none")))
        username_status = "ready"

    password_status = "working"
    while password_status != "ready":
        clear()
        for i in progress_visual:
            console.print(i)

        console.print(Panel(Text.assemble(("Enter a password (1)", "bold cyan")),
                            style="bold cyan", border_style="bold cyan"))
        password = Prompt.ask(Text.assemble(("╰→", "bold red")), password=True)
        console.print("Checking password...", style="bold yellow")
        await asyncio.sleep(0.3)
        # Checks
        if " " in password:
            clear()
            console.print(Panel(Text.assemble(("Can not have space in password", "bold cyan")),
                                style="bold red", border_style="bold red"))
            await asyncio.sleep(2.1)
            clear()
            continue
        if len(password) < 8:
            clear()
            console.print(Panel(Text.assemble(("Password must be at least 8 characters long",
                                               "bold cyan")), style="bold red", border_style="bold red"))
            await asyncio.sleep(2.1)
            clear()
            continue
        if len(password) > 32:
            clear()
            console.print(Panel(Text.assemble(("Password can not be any longer than 32 characters",
                                               "bold cyan")), style="bold red", border_style="bold red"))
            await asyncio.sleep(2.1)
            clear()
            continue
        if not re.search("[a-z]", password):
            clear()
            console.print(Panel(Text.assemble(("Password must contain at least one lowercase character",
                                               "bold cyan")), style="bold red", border_style="bold red"))
            await asyncio.sleep(2.1)
            clear()
            continue
        if not re.search("[A-Z]", password):
            clear()
            console.print(Panel(Text.assemble(("Password must contain at least one uppercase character",
                                               "bold cyan")), style="bold red", border_style="bold red"))
            await asyncio.sleep(2.1)
            clear()
            continue
        if not re.search("[A-Z]", password):
            clear()
            console.print(Panel(Text.assemble(("Password must contain at least one uppercase character",
                                               "bold cyan")), style="bold red", border_style="bold red"))
            await asyncio.sleep(2.1)
            clear()
            continue
        if not re.search("[0-9]", password):
            clear()
            console.print(Panel(Text.assemble(("Password must contain at least one digit", "bold cyan")),
                                style="bold red", border_style="bold red"))
            await asyncio.sleep(2.1)
            clear()
            continue
        for i in password:
            if i not in printable_chars:
                clear()
                console.print(Panel(Text.assemble(("Password can only contain letters from the ASCII table",
                                                   "bold cyan")), style="bold red", border_style="bold red"))
                await asyncio.sleep(2.1)
                clear()
                continue
        clear()
        for i in progress_visual:
            console.print(i)
        console.print(Panel(Text.assemble(("Enter a password (1)", "bold cyan")),
                            style="bold cyan", border_style="bold cyan"))
        console.print(Text.assemble(("╰→", "bold red")))

        console.print(Panel(Text.assemble(("Repeat password (2)", "bold cyan")),
                            style="bold cyan", border_style="bold cyan"))
        password2 = Prompt.ask(Text.assemble(("╰→", "bold red")), password=True)

        if password != password2:
            clear()
            console.print(Panel(Text.assemble(("Passwords did not match", "bold cyan")),
                                style="bold red", border_style="bold red"))
            await asyncio.sleep(2.1)
            clear()
            continue

        progress_visual.append(Panel(Text.assemble(("Password ☺", "bold green")),
                                     style="bold green", border_style="bold green"))
        progress_visual.append(Text.assemble(("╰→", "bold red"), ("HIDDEN", "bold yellow")))
        password_status = "ready"
    clear()
    for i in progress_visual:
        console.print(i)

    a = await create_account(username, password2)

    console.print(Panel(Text.assemble(("Successfully created an account!")), style="bold green", border_style="green"))
    await asyncio.sleep(2.1)
    clear()
    return a


async def log_in():
    status = "working"
    while status != "done":
        clear()
        console.print(Panel(Text.assemble(("Username", "bold purple")),
                            style="bold magenta", border_style="bold purple"))
        username = Prompt.ask(Text.assemble(("╰→", "bold red")))

        console.print(Panel(Text.assemble(("Password", "bold purple")), style="bold cyan", border_style="bold cyan"))
        password = Prompt.ask(Text.assemble(("╰→", "bold red")), password=True)

        console.print("Validating login...", style="bold yellow")
        user_data = await get_user_data(username)
        

        if user_data is None:
            clear()
            console.print(Panel(Text.assemble(("Invalid username or password. Please try again.",
                                               "bold red")), style="bold red", border_style="bold red"))
            await asyncio.sleep(2.1)
            continue

        if not sha256(bytes(password, "utf-8")).hexdigest() == remove_salt(user_data[1]):
            clear()
            console.print(Panel(Text.assemble(("Invalid username or password. Please try again.",
                                               "bold red")), style="bold red", border_style="bold red"))
            await asyncio.sleep(2.1)
            continue
        clear()
        console.print(Panel(Text.assemble(("Success! Logged in as " + username, "bold green")),
                            style="bold green", border_style="green"))
        await asyncio.sleep(2.1)
        clear()
        prefs = Preferences()
        new_pref_dict = {
            "Name Colour": user_data[2],
            "Message Colour": user_data[3],
            "Border Colour": user_data[4],
            "Message Border Colour": user_data[5]
        }
        prefs.preference_dict = new_pref_dict
        return User(username, hash_pass(password), prefs)


async def login():
    choice = utils.make_style_prompt(choices=["Log in", "Sign up"], prompt_msg="Please log-in/sign up:",
                                     main_style="bold purple", frame_border_style="bold cyan", frame_style="bold red")
    if choice == "Log in":
        return await log_in()
    if choice == "Sign up":
        return await sign_up()


async def exit_client():
    await sio.disconnect()
    exit()


def set_rooms(data):
    globals().update(ROOMS=data["rooms"])


async def get_rooms():
    await sio.emit("get_rooms", callback=set_rooms)


@sio.event
def set_user_data(data):
    if data["sid"] != sio.sid:
        return
    globals().update(LOGIN_DATA=data["info"])
    globals().update(WAIT_FOR_INFO=False)


async def update_user(user: User):
    prefs = user.preferences.preference_dict
    username = user.username
    password = user.hashed_pass
    
    success = False
    while not success:    
        try:
            await sio.emit("update_user", {"username": username, "passwrd": password, "prefs": prefs})
            success = True
        except socketio.exceptions.BadNamespaceError:
            try:
                await sio.connect(SERVER_ADDRESS)
            except socketio.exceptions.ConnectionError:
                feedback = Panel("Lost connection and could not reconnect. Please check your internet connection and restart the program.", style="bold red", border_style="bold red")
                console.print(feedback)
            continue



async def get_user_data(username):
    globals().update(WAIT_FOR_INFO=True)
    success = False
    while not success:    
        try:
            await sio.emit("get_user_data", {"to_access": username, "sid": sio.sid})
            success = True
        except socketio.exceptions.BadNamespaceError:
            try:
                await sio.connect(SERVER_ADDRESS)
            except socketio.exceptions.ConnectionError:
                feedback = Panel("Lost connection and could not reconnect. Please check your internet connection and restart the program.", style="bold red", border_style="bold red")
                console.print(feedback)
            continue

            
    
    while True:
        await asyncio.sleep(0.7)
        global WAIT_FOR_INFO
        if not WAIT_FOR_INFO:
            break
    global LOGIN_DATA
    info = LOGIN_DATA
    globals().update(LOGIN_DATA=0)
    return info


async def save_user(username, password, pref_dict):
    success = False
    while not success:
        await asyncio.sleep(0.1)
        try:
            await sio.emit("save_user", {"username": username, "password": password, "pref_dict": pref_dict})
            success = True
        except socketio.exceptions.BadNamespaceError:
            try:
                await sio.connect(SERVER_ADDRESS)
            except socketio.exceptions.ConnectionError:
                feedback = Panel("Lost connection and could not reconnect. Please check your internet connection and restart the program.", style="bold red", border_style="bold red")
                console.print(feedback)
            continue
    


def get_room_data():
    return ROOMS


@sio.event
async def connect():
    globals().update(CONNECTED=True)


@sio.event
async def disconnect():
    globals().update(ROOM_WORKS=False)
    globals().update(CONNECTED=False)


@sio.event
async def send_message(sid, data):
    return #print(f'[CLIENT]: message sent {sid}, data: {data}')


@sio.event
async def receive_message(data):
    global messages_to_show, ROOM, USERNAME
    if data["room_name"] == ROOM:
        if USERNAME != data["username"]:
            messages_to_show.append([data["username"], data["message"]])


async def main():
    console.print(Panel("Starting connection...", style="bold yellow", border_style="bold yellow"))
    try:
        await sio.connect(SERVER_ADDRESS)
    except socketio.exceptions.ConnectionError as e:
        feedback = Panel("Could not connect to server. Please check your internet connection and restart the program.", style="bold red", border_style="bold red")
        console.print(feedback)
        exit()

    console.print(Panel('Connected!', style="bold green", border_style="bold green"))
    console.print(Panel("Enjoy your stay!", style="bold green", border_style="bold green"))
    await asyncio.sleep(2)
    await rendering.load_box_animation(None)
    return await console_loop()


async def ping_server():
    await sio.emit("keep_alive")


async def console_loop(user=None):
    global messages_to_show
    clear()
    if user is None:
        action, user = await main_navigation.main_menu(logged_in=False, logged_in_as=None)
    if user is not None:
        action, user = await main_navigation.main_menu(logged_in=True, logged_in_as=user)
    
    if user is not None:
        await update_user(user)

    if action == "exit":
        if user is not None:
            await update_user(user)        
            await asyncio.sleep(1)
            print("Saving account preferences...")
        try:
            await sio.disconnect()
        except:
            pass
        await sio.wait()
        return
    if action == "login":
        user = await login()
        return await console_loop(user=user)
    globals().update(USERNAME=user.username)
    

    console.print(Panel("Enter the name of a box to join \nIf the box doesn't exist a new one will be created", style=user.preferences.preference_dict["Border Colour"], border_style=user.preferences.preference_dict["Border Colour"]))
    name = Prompt.ask(Text.assemble(("╰>", user.preferences.preference_dict["Border Colour"])))
    console.print(Panel(f"Joining {name}", style="green", border_style="green"))
    await sio.emit("join_room", {"username": user.username, "room_name": name, "sid": sio.sid})
    await asyncio.sleep(1)
    globals().update(ROOM=name)
    globals().update(ROOM_WORKS=True)
    clear()
        
        

    cancel_render = False
    while True:
        if not cancel_render:
            console.print(await rendering.render_menu_screen(await rendering.get_message_box_rows([], user)))
            console.print("Tips: Hold AltGr+Space to type, Hold AltGR+C to go back to main-menu.")
        else:
            cancel_render = False

        
        wait = True
        while wait:
            if not ROOM_WORKS: # Check if connection was lost to reconnect if it was try to reconnect.
                console.print(Panel("Your connection was lost.", style="bold yellow", border_style="bold yellow"))
                console.print(Panel("Reconnecting...", style="bold yellow", border_style="bold yellow"))


                loop_count = 0 # Store loop-count to cancel reconnect if it exceeds time-limit.
                feedback = Panel("Could not reconnect. Please check your internet connection and restart the program.", style="bold red", border_style="bold red")
                back_online = False
                
                reconnecting = True
                while reconnecting:
                    await asyncio.sleep(1)
                
                    loop_count += 1
                    if loop_count == 11:
                        break

                    try:
                        await sio.emit("join_room", {"username": user.username, "room_name": name})
                    except Exception as e:
                        continue
                
                    feedback = Panel("Back online!", style="bold green", border_style="bold green")
                    reconnecting = False
                    back_online = True
                    globals().update(ROOM_WORKS=True)
                console.print(feedback)

                if back_online:
                    clear()
                    console.print(await rendering.render_menu_screen(await rendering.get_message_box_rows([], user)))
                    console.print("Tips: Hold AltGr+Space to type, Hold AltGR+C to go back to main-menu.")

            
            if keyboard.is_pressed("alt gr+space"):
                event = "msg"
                break
            if keyboard.is_pressed("alt gr+c"):
                event = "return"
                break
            global messages_to_show
            if len(messages_to_show) != 0:
                clear()
                index_of_i = -1
                for i in messages_to_show:
                    index_of_i += 1
                    with Live("", refresh_per_second=14) as live:
                        render_user = User(i[0], "NotImportant", preferences=user.preferences)
                        await rendering.render_message(i[1], render_user, live=live)
                        messages_to_show.pop(index_of_i)
                clear()
                console.print(await rendering.render_menu_screen(await rendering.get_message_box_rows([], user)))
                console.print("Tips: Hold AltGr+Space to type, Hold AltGR+C to go back to main-menu.")
                
            await asyncio.sleep(0.2)
        if event == "msg":
            clear()
            message = await rendering.prompt(user)
            clear()
            if not ROOM_WORKS: # Check if connection was lost to reconnect if it was try to reconnect.
                console.print(Panel("Your connection was lost.", style="bold yellow", border_style="bold yellow"))
                console.print(Panel("Reconnecting...", style="bold yellow", border_style="bold yellow"))


                loop_count = 0 # Store loop-count to cancel reconnect if it exceeds time-limit.
                feedback = Panel("Could not reconnect. Please check your internet connection and restart the program.", style="bold red", border_style="bold red")
                back_online = False
                
                reconnecting = True
                while reconnecting:
                    await asyncio.sleep(1)
                
                    loop_count += 1
                    if loop_count == 11:
                        break

                    try:
                        await sio.emit("join_room", {"username": user.username, "room_name": name})
                    except Exception as e:
                        continue
                
                    feedback = Panel("Back online!", style="bold green", border_style="bold green")
                    reconnecting = False
                    back_online = True
                    globals().update(ROOM_WORKS=True)
                console.print(feedback)

                if back_online:
                    clear()
                    console.print(await rendering.render_menu_screen(await rendering.get_message_box_rows([], user)))
                    console.print("Tips: Hold AltGr+Space to type, Hold AltGR+C to go back to main-menu.")        
            await sio.emit("send_message", {"username": user.username, "message": message, "room_name": name})
            await asyncio.sleep(0.2)
            with Live("", refresh_per_second=14) as live:
                render_user = User(USERNAME, "NotImportant", preferences=user.preferences)
                await rendering.render_message(message, render_user, live=live)
                live.update(Text.assemble(await rendering.render_menu_screen(await rendering.get_message_box_rows([], user)), ("\nTips: Hold AltGr+Space to type, Hold AltGR+C to go back to main-menu.")))
            cancel_render = True
            
        if event == "return":
            await sio.emit("leave_room", {"username": user.username, "room_name": name})
            await asyncio.sleep(0.2)
            return await console_loop(user)

def start():
    try:
        asyncio.run(main())
        exit_anim()
    except RuntimeError:
        pass

if __name__ == "__main__":
    start()


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='thabox',
    version='0.1.1',
    author='The Jazzed Jerboas',
    author_email='thorvaldsenmikkel@gmail.com',
    description="""ThaBox is an open-source, terminal-based, retro-modern chat application where clients can connect to chatrooms and talk anonymously.""",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/MikeNoCap/ThaBox',
    project_urls = {
        "Bug Tracker": "https://github.com/MikeNoCap/ThaBox/issues"
    },
    license='MIT',
    packages=['thabox'],
    install_requires=["python-socketio-client~=1.1", "python-socketio~=5.3", "keyboard~=0.13", "rich~=10.6", "playsound", "uvicorn"],
    entry_points = {
        'console_scripts': ['thabox=thabox.client:start'],
    }
)


from rich.text import Text
from rich.prompt import Prompt
from rich.panel import Panel
from rich.screen import Screen
from rich.console import Console, RenderGroup
from rich.align import Align

try:
    from utils import User, Preferences, GoBack, clear
    from rendering import render_menu_screen, message_demo
    import login
    import credits
    import help as help_page
except ModuleNotFoundError:
    from thabox.utils import User, Preferences, GoBack, clear
    from thabox.rendering import render_menu_screen, message_demo
    import thabox.credits as credits
    import thabox.help as help_page

console = Console()

main_title = Text.assemble(
    (
        """
$$$$$$$$\\ $$\\
\\__$$  __|$$ |
   $$ |   $$$$$$$\\   $$$$$$\\
   $$ |   $$  __$$\\  \\____$$\\
   $$ |   $$ |  $$ | $$$$$$$ |
   $$ |   $$ |  $$ |$$  __$$ |
   $$ |   $$ |  $$ |\\$$$$$$$ |
   \\__|   \\__|  \\__| \\_______|\n
""",
        "bold magenta"),
    (
        """$$$$$$$\\
$$  __$$\\
$$ |  $$ | $$$$$$\  $$\   $$\\
$$$$$$$\ |$$  __$$\ \$$\ $$  |
$$  __$$\ $$ /  $$ | \$$$$  /
$$ |  $$ |$$ |  $$ | $$  $$<
$$$$$$$  |\$$$$$$  |$$  /\$$\\
\\_______/  \\______/ \\__/  \\__|
""",
        "bold cyan")
)


async def main_menu(logged_in: bool = False, logged_in_as=None):
    event = ""
    clear()
    selected = False
    if logged_in:
        options = {
            "Logged in as " + logged_in_as.username: 1,
            "Create/join a box": 2,
            "Preferences": 3,
            "Help/Info": 4,
            "Credits": 5,
            "Exit": 6
        }
    if not logged_in:
        options = {
            "Log in/Sign up": 1,
            "Help/Info": 2,
            "Credits": 3,
            "Exit": 4
        }

    hover_on = 1
    while not selected:
        rows = []
        rows_temp = [x.center(len("--------------------"), "|") for x in list(options)]
        if logged_in:
            logged_in_as_msg = rows_temp.pop(0)
        rows_temp[hover_on - 1] = Text.assemble(
            (rows_temp[hover_on - 1].center(len("--------------------"), "|"), "bold yellow"))

        rows.append(Text.assemble(("--------------------", "bold purple" if not logged_in else logged_in_as.preferences.preference_dict["Border Colour"])))
        for i in rows_temp:
            rows.append("")
            rows.append(i)
            rows.append("")
            rows.append(Text.assemble(("--------------------", "bold purple" if not logged_in else logged_in_as.preferences.preference_dict["Border Colour"])))

        # Fill up unused space.
        while len(rows) < 21:
            rows.append(Text.assemble(("--------------------", "bold purple" if not logged_in else logged_in_as.preferences.preference_dict["Border Colour"])))

        if logged_in:
            rows.insert(0, Text.assemble((logged_in_as_msg, "bold purple" if not logged_in else logged_in_as.preferences.preference_dict["Border Colour"])))

        console.print(await render_menu_screen(rows))
        if logged_in:
            what_to_do = Prompt.ask(Text.assemble(("1: Scroll up\n", "bold cyan"), ("2: Scroll down\n", "bold cyan"),
                                                  ("3: Select\n", "bold purple"), ("out: Log out\n", "bold red")),
                                    choices=["1", "2", "3", "out"], default="3")
        if not logged_in:
            what_to_do = Prompt.ask(Text.assemble(("1: Scroll up\n", "bold cyan"), ("2: Scroll down\n", "bold cyan"),
                                                  ("3: Select\n", "bold purple")), choices=["1", "2", "3"], default="3")
        if what_to_do == "1":
            if hover_on == 1:
                clear()
                continue
            hover_on -= 1
        if what_to_do == "2":
            if hover_on == len(rows_temp):
                clear()
                continue
            hover_on += 1
        if what_to_do == "3":
            if logged_in:
                if hover_on == 1:
                    try:
                        clear()
                        event = "box"
                        return event, logged_in_as  # Create/join box
                    except GoBack:
                        hover_on = 1
                        clear()
                        continue
                elif hover_on == 2:
                    try:
                        clear()
                        logged_in_as.preferences.settings()  # Preferences
                        hover_on = 1
                        clear()
                        continue
                    except GoBack:
                        hover_on = 1
                        clear()
                        continue
                elif hover_on == 3:
                    try:
                        clear()
                        help_page.help_start_page(logged_in_as)  # Help/Info
                        hover_on = 1
                        clear()
                        continue
                    except GoBack:
                        hover_on = 1
                        clear()
                        continue
                elif hover_on == 4:
                    try:
                        clear()
                        credits.credits_rework()  # Credits
                        hover_on = 1
                        clear()
                        continue
                    except GoBack:
                        hover_on = 1
                        clear()
                        continue
                elif hover_on == 5:
                    clear()
                    event = "exit"
                    return event, logged_in_as  # Replaced by exit function in client so the socket connection is properly closed.
                elif what_to_do == "out":
                    return await main_menu(logged_in=False, logged_in_as=None)
            if hover_on == 1:
                clear()
                event = "login"
                return event, None # User is None
            elif hover_on == 2:
                try:
                    clear()
                    help_page.help_start_page(logged_in_as)  # Help/Info
                    hover_on = 1
                    clear()
                    continue
                except GoBack:
                    hover_on = 1
                    clear()
                    continue
            elif hover_on == 3:
                try:
                    clear()
                    credits.credits_rework()  # Credits
                    hover_on = 1
                    clear()
                    continue
                except GoBack:
                    hover_on = 1
                    clear()
                    continue
            elif hover_on == 4:
                clear()
                event = "exit"
                return event, logged_in_as  # Replaced by exit function in client so the socket connection is properly closed.
        

        clear()






import keyboard
import time
import sys
import random
from utils import clear
from utils import GoBack
from utils import User
from rich import print
from rich.console import Console
from rich.align import Align
from rich.prompt import Prompt
from rich.panel import Panel
from rich.layout import Layout
from rich.text import Text
from rendering import render_chat_rooms, render_menu_screen

"""
'id, password, size, stype = create_box_tui(User, sessions_data)'
To run the create_box_tui - the variables can be changed (i.e. to a list), by 
returning a different value at lines 322 and 352. To be called on main.py, hover_on = 1

'join_box_tui(User, sessions_data)'
To run the join_box_tui - no variables returned, as the session id's, from sessions_data
is used to print and select a valid session. May need to be edited to return session_id if needed.
To be called on main.py, hover_on = 2
"""

console = Console()

"""
This will be passed into the join_box_tui() function when called. List(?) to be filled
from the database. After database is created, it needs to be implemented into this code.
currently a temp variable
"""

sessions_data = [[1234]]

global layout, colours, private_box_art, public_box_art

colour = {
    "box_default": "bold white",
    "box_hover": "bold yellow"
}

public_box_art = ("""
          +--+
          |  |
          |  |
          |  |
          |  |
          |  |
        +-+  +-+
   +-----\    /-------+
  /|      \  /       /|
 / |       \/       / |  
+--+---------------+  |
|                  |  |
|    PUBLIC BOX    |  +
|                  | /
+------------------+/
""")

private_box_art = ("""
         .------.
        / .----. \  
       _| |____| |_
     .'    ____    '.
     |    /    \    |  
     |    \____/    |
     '.____________.' 
   +------------------+
  /                  /|
 /                  / |  
+--+---------------+  |
|                  |  |
|    PRIVATE BOX   |  +
|                  | /
+------------------+/
""")

# Splitting the console to place the boxes side by side
layout = Layout()
layout.visible = True
layout.split_column(
    Layout(name="title"),
    Layout(name="boxes"),
    Layout(name="screen")
)

layout["title"].size = 5
layout["boxes"].size = 16
layout["screen"].size = 5

layout["boxes"].split_row(
    Layout(name="left"),
    Layout(name="right")
)

"""
temp values for testing

sessions_data = [
    {
        "room_name": "1234",
        "private" : True,
        "password" : "hello",
        "capacity" : "1",
        "room_owner" : "bob"
    },
    {
        "room_name": "2222",
        "private" : True,
        "password" : "zxcv",
        "capacity" : "1",
        "room_owner" : "lob"
    },
    {
        "room_name": "3333",
        "private" : True,
        "password" : "sdfg",
        "capacity" : "1",
        "room_owner" : "tob"
    },
    {
        "room_name": "4444",
        "private" : False,
        "password" : None,
        "capacity" : "1",
        "room_owner" : "mob"
    },
    {
        "room_name": "5555",
        "private" : False,
        "password" : None,
        "capacity" : "1",
        "room_owner" : "cob"
    },
]
"""


def is_already_an_id(sessions_data, session_id, server_type="all"):
    if not server_type:
        for elem in sessions_data:
            if not elem["private"]:
                if elem["room_name"] == session_id:
                    return True

    elif server_type == "all":
        for elem in sessions_data:
            if elem["room_name"] == session_id:
                return True

    else:
        for elem in sessions_data:
            if elem["private"]:
                if elem["room_name"] == session_id:
                    return True

    return False


def is_correct_password(sessions_data, password, session_id):
    for elem in sessions_data:
        if elem["room_name"] == session_id:
            if elem["password"] == password:
                return True
    return False


def layout_setup(select: str, text: str):  # DONE
    """
    Creates menu layout, with ASCII box art and navigation help text.
    """
    global layout, private_box_art, public_box_art, colour

    clear()
    layout["title"].update(Align(text, align="center"))
    layout["screen"].update(
        "\n\n[bold white]!   Use [bold magenta] right arrow[/] or [bold magenta]left arrow[/] to select a box.[/]"
        + "\n[bold white]!   [bold magenta]ENTER[/] to confirm your choice.[/]"
        + "\n[bold white]!   Press [bold magenta]BACKSPACE[/] to return to Menu.[/]")

    if select == "left":
        layout["right"].update(Align(Text(f"{private_box_art}", style=colour["box_default"]), align="center"))
        layout["left"].update(Align(Text(f"{public_box_art}", style=colour["box_hover"]), align="center"))
    else:
        layout["right"].update(Align(Text(f"{private_box_art}", style=colour["box_hover"]), align="center"))
        layout["left"].update(Align(Text(f"{public_box_art}", style=colour["box_default"]), align="center"))

    console.print(layout)


def tui_navigation(select: str, sessions_data):  # DONE
    """
    Returns which box is selected.
    """

    # Adds delay between key presses
    time.sleep(0.2)
    while True:
        if keyboard.is_pressed("backspace"):
            raise GoBack

        if keyboard.is_pressed("left_arrow"):
            if select != "left":
                select = "left"
                return select
            continue

        if keyboard.is_pressed("right_arrow"):
            if select != "right":
                select = "right"
                return select
            continue

        if keyboard.is_pressed("enter"):
            if select == "left":
                select = "public"
                return select

            elif select == "right":
                select = "private"
                return select


def create_session_id(sessions_data):  # DONE
    """
    Adds leading 0's if necessary and returns a unique session_id,
    by comparing it to taken id's.
    """
    num: int = random.randint(1, 1000)
    session_id: str = ("0" * (4 - len(str(num)))) + str(num)
    if is_already_an_id(sessions_data, session_id):
        return create_session_id(sessions_data)

    return str(session_id)


def enter_session_id(prompt: str, alignment: str, password_prompt, sessions_data):
    """
    Prompts user to enter a valid session ID
    """
    given_id = console.input(prompt)

    if given_id.lower() == "back":
        return join_box_tui(User, sessions_data)

    elif (len(given_id) == 4) and (given_id.isdigit() is True):
        if is_already_an_id(sessions_data, given_id, password_prompt):
            if password_prompt:
                enter_password(sessions_data, password_prompt, given_id)

            else:
                console.print(Align("\n✔️   Joining ThaBox...", align=alignment))
                time.sleep(0.5)
                clear()
                return given_id

        else:
            console.print(Align("❌   The room you are trying to join doesn't exist\n", align=alignment))
            time.sleep(0.5)
            return enter_session_id(prompt, alignment, password_prompt, sessions_data)

    else:
        console.print(Align("❌   Session ID's can only be 4 digit numbers!\n", align=alignment))
        time.sleep(0.5)
        return enter_session_id(prompt, alignment, password_prompt, sessions_data)


def enter_password(sessions_data, prompt, session_id):
    """
    Checks if entered password is correct.
    """
    time.sleep(0.3)  # Input delay
    given_password = console.input(prompt)
    if is_correct_password(sessions_data, given_password, session_id):
        console.print(Align("\n✔️   Joining ThaBox...", align="center"))
        time.sleep(1)
        clear()
        return session_id, given_password

    else:
        console.print(Align("❌   Incorrect Password!", align="center"))
        return enter_password(sessions_data, prompt, session_id)


def enter_room_size(user):  # DONE
    """
    Validates the user input, so they enter a correct size, and 
    returns the room_size.
    """
    size = console.input(" " * ((console.width // 2) - 14) + "[bold red]Enter room size (2 - 6):   [/]")

    if size.lower() == "back":
        return create_box_tui(user, sessions_data)

    if size == "2" or size == "3" or size == "4" or size == "5" or size == "6":
        return str(size)

    else:
        console.print(Align("❌   Enter a valid number!\n", align="center"))
        return enter_room_size(user)


def join_box_tui(user: User, sessions_data, select="left"):
    if select == "public":
        """
        PLAN - print only 4 session at once. Users can navigate this using arrow keys.
        They type the session id of the room they wan to join, and then they join the room.
        DATA NEEDED - public session id, max user count for the room, current amount of users
        in the room. 
        """
        clear()

        console.print(Align("\n█▀█ █ █ █▄▄ █   █ █▀▀   █▄▄ █▀█ ▀▄▀ █▀▀ █▀\n"
                            + "█▀▀ █▄█ █▄█ █▄▄ █ █▄▄   █▄█ █▄█ █ █ ██▄ ▄█\n",
                            align="center"),
                      style=user.preferences.preference_dict['Border Colour'])

        for elem in sessions_data:
            if not elem["private"]:
                console.print(Align(Panel("Session ID = " + elem["room_name"]
                                          + "\nOwner = " + elem["room_owner"]
                                          + "\nCapacity = " + elem["capacity"], expand=False), align="center"),
                              style="bold white")
        console.print(
            "\n\n[bold white]!   Use [bold magenta] right arrow[/] or [bold magenta]left arrow[/] to find more rooms.[/]\n"
            + "[bold white]!   [bold magenta]SPACE[/] to type in a session ID.[/]\n"
            + "[bold white]!   Press [bold magenta]BACKSPACE[/] to go back.[/]")

        time.sleep(0.2)
        while True:
            if keyboard.is_pressed("backspace"):
                select = "left"
                return join_box_tui(user, sessions_data, select)

            if keyboard.is_pressed("space"):
                console.print()
                chatroom_name = enter_session_id(
                    "[bold red]>   Enter the session ID or type 'BACK' to go back:[/]   ", "left", False, sessions_data)

                return chatroom_name
                """
                user needs to be redirected to another chat room
                """

    elif select == "private":
        clear()
        console.print(Align("\n█▀█ █▀█ █ █ █ ▄▀█ ▀█▀ █▀▀   █▄▄ █▀█ ▀▄▀\n"
                            + "█▀▀ █▀▄ █ ▀▄▀ █▀█  █  ██▄   █▄█ █▄█ █ █\n", align="center"),
                      style=user.preferences.preference_dict["Border Colour"])

        console.print(Align("\nType [bold magenta]BACK[/] in the Session ID field to go back.\n", align="center"))

        chatroom_name, room_password = enter_session_id((" " * ((console.width // 2) - 14) + "[bold red]Enter the session ID:[/]   "), "center",
                                                        (str("\n" + " " * ((console.width // 2) - 17) + "[bold red]Enter the room password:[/]   ")), sessions_data)

        return chatroom_name
        """
        user needs to be redirected to the chat room
        """

    else:
        """
        JOIN BOX MENU
        """
        layout_setup(select,
                     f"[{user.preferences.preference_dict['Border Colour']}]\n\n  █ █▀█ █ █▄ █   █▄▄ █▀█ ▀▄▀\n█▄█ █▄█ █ █ ▀█   █▄█ █▄█ █ █[/]")  # select, "JOIN BOX"
        select = tui_navigation(select, sessions_data)
        join_box_tui(user, sessions_data, select)


def create_box_tui(user: User, sessions_data, select="left"):
    """
    Allows user to create thier own box - can set the room_size
    and add a password, which are both returned along with a random,
    unique session id.
    """

    if select == "public":
        clear()
        public_session_id = create_session_id(sessions_data)

        console.print(Align("\n█▄▄ █▀█ ▀▄▀   █▀ █▀▀ ▀█▀ ▀█▀ █ █▄ █ █▀▀ █▀\n"
                            + "█▄█ █▄█ █ █   ▄█ ██▄  █   █  █ █ ▀█ █▄█ ▄█\n", align="center"),
                      style=user.preferences.preference_dict['Border Colour'])
        console.print(Align("Type [bold magenta]BACK[/] in any of the fields to go back.\n", align="center"))
        console.print(Align(f"[bold red]Session ID:[/]   {public_session_id}", align="center"))

        room_size = enter_room_size(user)

        console.print(Align("\n✔️   Creating ThaBox...", align="center"))
        time.sleep(0.5)

        is_private = False
        password = None

        return "create", user, public_session_id, password, room_size, is_private

    elif select == "private":
        clear()
        private_session_id = create_session_id(sessions_data)

        console.print(Align("\n█▄▄ █▀█ ▀▄▀   █▀ █▀▀ ▀█▀ ▀█▀ █ █▄ █ █▀▀ █▀\n"
                            + "█▄█ █▄█ █ █   ▄█ ██▄  █   █  █ █ ▀█ █▄█ ▄█\n", align="center"),
                      style=user.preferences.preference_dict['Border Colour'])
        console.print(Align("Type [bold magenta]BACK[/] in any of the fields to go back.\n", align="center"))
        console.print(Align(f"[bold red]Session ID:[/]   {private_session_id}", align="center"))

        while True:
            time.sleep(0.2)
            password = console.input(" " * ((console.width // 2) - 12) + "[bold red]Create a Password:   [/]")
            if password.lower() == "back":
                return create_box_tui(user, sessions_data)
            elif password == "":
                console.print(Align("❌   Password can't be blank!\n", align="center"))
            elif " " in password:
                console.print(Align("❌   Password can't have spaces!\n", align="center"))
            elif len(password) < 8:
                console.print(Align("❌   Password must have at least 8 characters!\n", align="center"))

            else:
                room_size = enter_room_size(user)
                break

        console.print(Align("\n✔️   Creating ThaBox...", align="center"))
        time.sleep(0.5)

        is_private = True
        return "create", user, private_session_id, password, room_size, is_private

    else:
        """
        CREATE BOX MENU
        """
        layout_setup(select,
                     f"[{user.preferences.preference_dict['Border Colour']}]\n\n█▀▀ █▀█ █▀▀ ▄▀█ ▀█▀ █▀▀   █▄▄ █▀█ ▀▄▀\n█▄▄ █▀▄ ██▄ █▀█  █  ██▄   █▄█ █▄█ █ █[/]")  # select, "CREATE BOX"

        select = tui_navigation(select, sessions_data)
        return create_box_tui(user, sessions_data, select)


def chat_rooms_scroll(logged_in_as: User):
    clear()
    selected = False
    options = {
        "Never": 1,
        "Gonna": 2,
        "Give": 3,
        "You": 4,
        "Up": 5,
        "Never": 6,
        "Gonna": 7,
        'Let': 8,
        'You': 9,
        'Down': 10,
    }

    hover_on = 1
    while not selected:
        rows = []
        rows_temp = [x.center(len("--------------------"), "|") for x in list(options)]

        rows_temp[hover_on - 1] = Text.assemble(
            (rows_temp[hover_on - 1].center(len("--------------------"), "|"), "bold yellow"))

        rows.append(Text.assemble(("--------------------", "bold blue")))
        for i in rows_temp:
            rows.append("")
            rows.append(i)
            rows.append("")
            rows.append(Text.assemble(("--------------------", "bold blue")))

        # Fill up unused space.
        while len(rows) < 21:
            rows.append(Text.assemble(("--------------------", "bold red")))

        console.print(render_menu_screen(rows))

        what_to_do = Prompt.ask(Text.assemble(("1: Scroll up\n", "bold cyan"), ("2: Scroll down\n", "bold cyan"),
                                              ("3: Select\n", "bold purple"), ("back: Back\n", "bold red")),
                                choices=["1", "2", "3", "back"], default="3")

        if what_to_do == "1":
            if hover_on == 1:
                clear()
                continue
            hover_on -= 1
        if what_to_do == "2":
            if hover_on == len(rows_temp):
                clear()
                continue
            hover_on += 1
        if what_to_do == "3":
            pass

        if what_to_do == "back":
            raise GoBack

        clear()


def ask_for_session():
    session = Prompt.ask("Please enter a session id")
    return session


def is_valid_session(session_id):
    """
    Use this function to test whether a given session id is valid or not.
    """
    for session in sessions_data:
        if session["room_name"] == session_id:
            return True
        else:
            continue
    return False


if __name__ == "__main__":
    chat_rooms_scroll({'Rick', 'Roll', 'Lol', })
